@@ -14,23 +14,34 @@ function torbutton_prefs_set_field_attributes(doc)
     doc.getElementById('torbutton_panelStyle').setAttribute("disabled", !doc.getElementById('torbutton_displayStatusPanel').checked);
     doc.getElementById('torbutton_panelStyleText').setAttribute("disabled", !doc.getElementById('torbutton_displayStatusPanel').checked);
     doc.getElementById('torbutton_panelStyleIcon').setAttribute("disabled", !doc.getElementById('torbutton_displayStatusPanel').checked);
-    // Privoxy is always recommended for Firefoxes not support socks_remote_dns
-    if (!torbutton_check_socks_remote_dns()) {
+
+    // Privoxy is always recommended for Firefoxes not supporting socks_remote_dns
+    if (doc.getElementById('torbutton_transparentTor').selected) {
+        doc.getElementById('torbutton_settingsMethod').value = 'transparent';
+    } else if (!torbutton_check_socks_remote_dns()) {
       doc.getElementById('torbutton_usePrivoxy').setAttribute("disabled", true);
     } else {
       doc.getElementById('torbutton_usePrivoxy').setAttribute("disabled", doc.getElementById('torbutton_settingsMethod').value != 'recommended');
     }
-    var proxy_port;
-    var proxy_host;
-    if (doc.getElementById('torbutton_usePrivoxy').checked) {
-        proxy_host = '127.0.0.1';
-        proxy_port = 8118;
-    } else {
-        proxy_host = '';
-        proxy_port = 0;
-    }
 
     if (doc.getElementById('torbutton_settingsMethod').value == 'recommended') {
+        var proxy_port;
+        var proxy_host;
+        if (torbutton_has_good_socks()) {
+          doc.getElementById('torbutton_usePrivoxy').checked = false;
+          doc.getElementById('torbutton_usePrivoxy').setAttribute("disabled", true);
+          proxy_host = '';
+          proxy_port = 0;
+        } else {
+          if (doc.getElementById('torbutton_usePrivoxy').checked) {
+            proxy_host = '127.0.0.1';
+            proxy_port = 8118;
+          } else {
+            proxy_host = '';
+            proxy_port = 0;
+          }
+        }
+
         torbutton_log(2, "using recommended settings");
         if (!torbutton_check_socks_remote_dns()) {
             doc.getElementById('torbutton_httpProxy').value = proxy_host;
@@ -52,9 +63,39 @@ function torbutton_prefs_set_field_attributes(doc)
             doc.getElementById('torbutton_gopherProxy').value = '';
             doc.getElementById('torbutton_gopherPort').value = 0;
         }
-        doc.getElementById('torbutton_socksHost').value = '127.0.0.1';
-        doc.getElementById('torbutton_socksPort').value = 9050;
+
+        var environ = Components.classes["@mozilla.org/process/environment;1"]
+                   .getService(Components.interfaces.nsIEnvironment);
+
+        if (environ.exists("TOR_SOCKS_PORT")) {
+          doc.getElementById('torbutton_socksPort').value = parseInt(environ.get("TOR_SOCKS_PORT"));
+        } else {
+          doc.getElementById('torbutton_socksPort').value = 9050;
+        }
+
+        if (environ.exists("TOR_SOCKS_HOST")) {
+          doc.getElementById('torbutton_socksHost').value = environ.get("TOR_SOCKS_HOST");
+        } else {
+          doc.getElementById('torbutton_socksHost').value = '127.0.0.1';
+        }
 
+        doc.getElementById('torbutton_httpProxy').disabled = true;
+        doc.getElementById('torbutton_httpPort').disabled = true;
+        doc.getElementById('torbutton_httpsProxy').disabled = true;
+        doc.getElementById('torbutton_httpsPort').disabled = true;
+        doc.getElementById('torbutton_ftpProxy').disabled = true;
+        doc.getElementById('torbutton_ftpPort').disabled = true;
+        doc.getElementById('torbutton_gopherProxy').disabled = true;
+        doc.getElementById('torbutton_gopherPort').disabled = true;
+        doc.getElementById('torbutton_socksHost').disabled = true;
+        doc.getElementById('torbutton_socksPort').disabled = true;
+        doc.getElementById('torbutton_socksGroup').disabled = true;
+        doc.getElementById('torbutton_noProxiesOn').disabled = true;
+    } else if (doc.getElementById('torbutton_settingsMethod').value == 'transparent') {
+        // Mr. Larry was so lazy when he wrote the rest of this code
+        torbutton_log(2, "not using recommended settings");
+        torbutton_log(2, "using transparent settings");
+        doc.getElementById('torbutton_usePrivoxy').setAttribute("disabled", true);
         doc.getElementById('torbutton_httpProxy').disabled = true;
         doc.getElementById('torbutton_httpPort').disabled = true;
         doc.getElementById('torbutton_httpsProxy').disabled = true;
@@ -68,6 +109,7 @@ function torbutton_prefs_set_field_attributes(doc)
         doc.getElementById('torbutton_socksGroup').disabled = true;
         doc.getElementById('torbutton_noProxiesOn').disabled = true;
     } else {
+        torbutton_log(2, "using transparent settings");
         doc.getElementById('torbutton_httpProxy').disabled = false;
         doc.getElementById('torbutton_httpPort').disabled = false;
         doc.getElementById('torbutton_httpsProxy').disabled = false;
@@ -144,6 +186,11 @@ function torbutton_prefs_init(doc) {
     }
     doc.getElementById('torbutton_noProxiesOn').value = o_torprefs.getCharPref('no_proxies_on');
 
+    // Transparent Torification magic
+    if (o_torprefs.getBoolPref('saved.transparentTor')) {
+        doc.getElementById('torbutton_settingsMethod').selectedItem = doc.getElementById('torbutton_transparentTor');
+    }
+
     // doc.getElementById('torbutton_warnUponExcludedSite').checked = o_torprefs.getBoolPref('prompt_before_visiting_excluded_sites');
 
     doc.getElementById('torbutton_disablePlugins').checked = o_torprefs.getBoolPref('no_tor_plugins');
@@ -168,7 +215,7 @@ function torbutton_prefs_init(doc) {
         o_torprefs.setBoolPref('cookie_jars', false);
         o_torprefs.setBoolPref('dual_cookie_jars', false);
         o_torprefs.setBoolPref('clear_cookies', true); 
-
+        o_torprefs.setBoolPref('cookie_protections',false);
         o_torprefs.setBoolPref('tor_memory_jar', true);
         o_torprefs.setBoolPref('nontor_memory_jar', true);
         doc.getElementById('torbutton_torMemoryJar').disabled = true;
@@ -179,26 +226,34 @@ function torbutton_prefs_init(doc) {
         o_torprefs.setBoolPref('cookie_jars', true);
         o_torprefs.setBoolPref('dual_cookie_jars', false);
         o_torprefs.setBoolPref('clear_cookies', false);
-
+        o_torprefs.setBoolPref('cookie_protections',false);        
         o_torprefs.setBoolPref('tor_memory_jar', true);
         doc.getElementById('torbutton_torMemoryJar').disabled = true;
+        doc.getElementById('torbutton_nonTorMemoryJar').disabled = false;        
+    } else if(o_torprefs.getBoolPref('cookie_protections')) {
+        doc.getElementById('torbutton_cookieGroup').selectedItem =
+            doc.getElementById('torbutton_cookieProtections');
+        o_torprefs.setBoolPref('cookie_jars', false);
+        o_torprefs.setBoolPref('dual_cookie_jars', true);
+        o_torprefs.setBoolPref('clear_cookies', false); 
+        doc.getElementById('torbutton_torMemoryJar').disabled = false;
         doc.getElementById('torbutton_nonTorMemoryJar').disabled = false;
-    } else if(o_torprefs.getBoolPref('dual_cookie_jars')) {
+    } 
+    else if(o_torprefs.getBoolPref('dual_cookie_jars')) {
         doc.getElementById('torbutton_cookieGroup').selectedItem =
             doc.getElementById('torbutton_dualCookieJars');
         o_torprefs.setBoolPref('cookie_jars', false);
         o_torprefs.setBoolPref('dual_cookie_jars', true);
         o_torprefs.setBoolPref('clear_cookies', false); 
-
+        o_torprefs.setBoolPref('cookie_protections',false);
         doc.getElementById('torbutton_torMemoryJar').disabled = false;
         doc.getElementById('torbutton_nonTorMemoryJar').disabled = false;
-    } else {
+    }  else {
         doc.getElementById('torbutton_cookieGroup').selectedItem =
             doc.getElementById('torbutton_mmmCookies');
         o_torprefs.setBoolPref('cookie_jars', false);
         o_torprefs.setBoolPref('dual_cookie_jars', false);
         o_torprefs.setBoolPref('clear_cookies', false); 
-
         o_torprefs.setBoolPref('tor_memory_jar', false);
         o_torprefs.setBoolPref('nontor_memory_jar', false);
         doc.getElementById('torbutton_torMemoryJar').disabled = true;
@@ -231,6 +286,7 @@ function torbutton_prefs_init(doc) {
             doc.getElementById('torbutton_restoreNonTor');
     }
 
+    /*
     switch(o_torprefs.getIntPref('startup_state')) {
         case 0: // non-tor
             doc.getElementById("torbutton_startupStateGroup").selectedItem =
@@ -244,8 +300,11 @@ function torbutton_prefs_init(doc) {
             doc.getElementById("torbutton_startupStateGroup").selectedItem =
                 doc.getElementById('torbutton_startPrevious');
             break;
-    }
-
+    }*/
+    
+    if ((doc.getElementById('torbutton_refererSpoofGroup').selectedIndex = o_torprefs.getIntPref('refererspoof'))==4)
+      doc.getElementById('torbutton_CustomRef').value = o_torprefs.getCharPref('customref');
+    
     doc.getElementById('torbutton_torSessionStore').checked = !o_torprefs.getBoolPref('notor_sessionstore');
     doc.getElementById('torbutton_nonTorSessionStore').checked = !o_torprefs.getBoolPref('nonontor_sessionstore');
 
@@ -263,7 +322,7 @@ function torbutton_prefs_init(doc) {
     doc.getElementById('torbutton_closeTor').checked = o_torprefs.getBoolPref('close_tor');
     doc.getElementById('torbutton_closeNonTor').checked = o_torprefs.getBoolPref('close_nontor');
     doc.getElementById('torbutton_setUagent').checked = o_torprefs.getBoolPref('set_uagent');
-    doc.getElementById('torbutton_noReferer').checked = o_torprefs.getBoolPref('disable_referer');
+    doc.getElementById('torbutton_refererSpoofGroup').selectedIndex = o_torprefs.getIntPref('refererspoof');
     doc.getElementById('torbutton_spoofEnglish').checked = o_torprefs.getBoolPref('spoof_english');
     doc.getElementById('torbutton_clearHttpAuth').checked = o_torprefs.getBoolPref('clear_http_auth');
     doc.getElementById('torbutton_blockJSHistory').checked = o_torprefs.getBoolPref('block_js_history');
@@ -290,6 +349,10 @@ function torbutton_prefs_init(doc) {
             doc.getElementById("torbutton_searchEngine").selectedItem =
                 doc.getElementById('torbutton_engine4');
             break;
+        case 5:
+            doc.getElementById("torbutton_searchEngine").selectedItem =
+                doc.getElementById('torbutton_engine5');
+            break;
 
     }
     doc.getElementById('torbutton_noCaptcha').checked = o_torprefs.getBoolPref('dodge_google_captcha');
@@ -317,21 +380,32 @@ function torbutton_cookie_update(doc) {
         doc.getElementById('torbutton_nonTorMemoryJar').checked = true;
         doc.getElementById('torbutton_torMemoryJar').disabled = true;
         doc.getElementById('torbutton_nonTorMemoryJar').disabled = true;
+
     } else if(doc.getElementById('torbutton_cookieGroup').selectedItem
             == doc.getElementById('torbutton_cookieJars')) {
         doc.getElementById('torbutton_torMemoryJar').checked = true;
         doc.getElementById('torbutton_torMemoryJar').disabled = true;
         doc.getElementById('torbutton_nonTorMemoryJar').disabled = false;
+
     } else if(doc.getElementById('torbutton_cookieGroup').selectedItem
             == doc.getElementById('torbutton_dualCookieJars')) {
         doc.getElementById('torbutton_torMemoryJar').disabled = false;
         doc.getElementById('torbutton_nonTorMemoryJar').disabled = false;
+
     } else if(doc.getElementById('torbutton_cookieGroup').selectedItem
             == doc.getElementById('torbutton_mmmCookies')) {
         doc.getElementById('torbutton_torMemoryJar').checked = false;
         doc.getElementById('torbutton_nonTorMemoryJar').checked = false;
         doc.getElementById('torbutton_torMemoryJar').disabled = true;
         doc.getElementById('torbutton_nonTorMemoryJar').disabled = true;
+
+    } else if(doc.getElementById('torbutton_cookieGroup').selectedItem
+            == doc.getElementById('torbutton_cookieProtections')) {
+        doc.getElementById('torbutton_torMemoryJar').checked = o_torprefs.getBoolPref('tor_memory_jar');
+        doc.getElementById('torbutton_nonTorMemoryJar').checked = o_torprefs.getBoolPref('nontor_memory_jar');
+        doc.getElementById('torbutton_torMemoryJar').disabled = false;
+        doc.getElementById('torbutton_nonTorMemoryJar').disabled = false;
+
     }
 }
 
@@ -365,7 +439,7 @@ function torbutton_prefs_save(doc) {
     o_torprefs.setIntPref('gopher_port',      doc.getElementById('torbutton_gopherPort').value);
     o_torprefs.setCharPref('socks_host',      doc.getElementById('torbutton_socksHost').value);
     o_torprefs.setIntPref('socks_port',       doc.getElementById('torbutton_socksPort').value);
-
+    o_torprefs.setIntPref('refererspoof',doc.getElementById('torbutton_refererSpoofGroup').selectedIndex);
     if(doc.getElementById('torbutton_socksGroup').selectedItem ==
             doc.getElementById('torbutton_socksv4')) {
         o_torprefs.setIntPref('socks_version', 4); 
@@ -375,7 +449,26 @@ function torbutton_prefs_save(doc) {
     }
 
     o_torprefs.setCharPref('no_proxies_on',      doc.getElementById('torbutton_noProxiesOn').value);
-    
+
+    o_torprefs.setBoolPref('saved.transparentTor', doc.getElementById('torbutton_transparentTor').selected);
+    if (o_torprefs.getBoolPref('saved.transparentTor')) {
+        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
+            .getService(Components.interfaces.nsIWindowMediator);
+        var chrome = wm.getMostRecentWindow("navigator:browser");
+        var ret = chrome.torbutton_test_settings();
+        if (ret != 4) {
+            var warning = strings.GetStringFromName("torbutton.popup.test.failure");
+            window.alert(warning);
+        } else {
+            o_torprefs.setBoolPref('tor_enabled', true);
+            // In theory this is where we unlock some things that are now "safe"
+            // Unleash Flash on Tor users
+            // Ignore any proxy settings that the user has set
+            // etc etc etc - pde will be very happy
+        }
+    }
+
+
     if (doc.getElementById('torbutton_settingsMethod').value == 'custom') {
         // XXX: Is this even needed anymore? We don't read the
         // custom prefs at all it seems..
@@ -399,7 +492,7 @@ function torbutton_prefs_save(doc) {
         }
     }
     // o_torprefs.setBoolPref('prompt_before_visiting_excluded_sites', doc.getElementById('torbutton_warnUponExcludedSite').checked);
-
+  o_torprefs.setIntPref('refererspoof', doc.getElementById('torbutton_refererSpoofGroup').selectedIndex);
     o_torprefs.setBoolPref('no_tor_plugins', doc.getElementById('torbutton_disablePlugins').checked);
     o_torprefs.setBoolPref('clear_history', doc.getElementById('torbutton_clearHistory').checked);
     o_torprefs.setBoolPref('kill_bad_js', doc.getElementById('torbutton_killBadJS').checked);
@@ -408,9 +501,11 @@ function torbutton_prefs_save(doc) {
 
     o_torprefs.setBoolPref('clear_cache', doc.getElementById('torbutton_clearCache').selected);
     o_torprefs.setBoolPref('block_cache', doc.getElementById('torbutton_blockCache').selected);
+
     o_torprefs.setBoolPref('clear_cookies', doc.getElementById('torbutton_clearCookies').selected);
     o_torprefs.setBoolPref('cookie_jars', doc.getElementById('torbutton_cookieJars').selected);
-    o_torprefs.setBoolPref('dual_cookie_jars', doc.getElementById('torbutton_dualCookieJars').selected);
+    o_torprefs.setBoolPref('dual_cookie_jars', doc.getElementById('torbutton_dualCookieJars').selected || doc.getElementById('torbutton_cookieProtections').selected);
+    o_torprefs.setBoolPref('cookie_protections', doc.getElementById('torbutton_cookieProtections').selected);
     o_torprefs.setBoolPref('disable_domstorage', doc.getElementById('torbutton_noDomStorage').checked);
     o_torprefs.setBoolPref('clear_http_auth', doc.getElementById('torbutton_clearHttpAuth').checked);
     o_torprefs.setBoolPref('block_js_history', doc.getElementById('torbutton_blockJSHistory').checked);
@@ -444,6 +539,7 @@ function torbutton_prefs_save(doc) {
             doc.getElementById('torbutton_restoreTorGroup').selectedItem ==
             doc.getElementById('torbutton_restoreTor'));
 
+    /*
     if(doc.getElementById('torbutton_startupStateGroup').selectedItem ==
             doc.getElementById('torbutton_startNonTor')) {
         o_torprefs.setIntPref('startup_state', 0);
@@ -452,7 +548,7 @@ function torbutton_prefs_save(doc) {
         o_torprefs.setIntPref('startup_state', 1);
     } else {
         o_torprefs.setIntPref('startup_state', 2);
-    }
+    }*/
 
     o_torprefs.setBoolPref('notor_sessionstore', !doc.getElementById('torbutton_torSessionStore').checked);
     o_torprefs.setBoolPref('nonontor_sessionstore', !doc.getElementById('torbutton_nonTorSessionStore').checked);
@@ -470,10 +566,11 @@ function torbutton_prefs_save(doc) {
     o_torprefs.setBoolPref('no_updates', doc.getElementById('torbutton_noUpdates').checked);
 
     o_torprefs.setBoolPref('set_uagent', doc.getElementById('torbutton_setUagent').checked);
-    o_torprefs.setBoolPref('disable_referer', doc.getElementById('torbutton_noReferer').checked);
     o_torprefs.setBoolPref('spoof_english', doc.getElementById('torbutton_spoofEnglish').checked);
 
     o_torprefs.setBoolPref('locked_mode', doc.getElementById('torbutton_lockedMode').checked);
+
+    o_torprefs.setIntPref('refererspoof',doc.getElementById('torbutton_refererSpoofGroup').selectedIndex);
     o_torprefs.setBoolPref('fix_google_srch', doc.getElementById('torbutton_fixGoogleSrch').checked);
     o_torprefs.setBoolPref('dodge_google_captcha', doc.getElementById('torbutton_noCaptcha').checked);
 
@@ -486,8 +583,11 @@ function torbutton_prefs_save(doc) {
     } else if(doc.getElementById('torbutton_searchEngine').selectedItem ==
             doc.getElementById('torbutton_engine3')) {
         o_torprefs.setIntPref('google_redir_url', 3);
-    } else {
+    } else if(doc.getElementById('torbutton_searchEngine').selectedItem ==
+            doc.getElementById('torbutton_engine4')) {
         o_torprefs.setIntPref('google_redir_url', 4);
+    } else {
+        o_torprefs.setIntPref('google_redir_url', 5);
     }
 
     /*
@@ -641,4 +741,9 @@ function torbutton_prefs_reset_defaults() {
 
     torbutton_log(4, "Preferences reset to defaults");
     torbutton_prefs_init(window.document);
+
+    // In all cases, force prefs to be synced to disk
+    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
+        .getService(Components.interfaces.nsIPrefService);
+    prefService.savePrefFile(null);
 }
